Files
project_6/cccl_upstream/cudax/test/common/group_testing.cuh
EngineX CI 56fd68e7dd [INFRA] Import NVIDIA/CCCL upstream as optimization reference library
CCCL (CUDA C++ Core Libraries) provides:
- CUB: device/block/warp-level GPU primitives (reduce, scan, sort, topk)
- Thrust: high-level parallel algorithms (transform_reduce, sort, scan)
- libcudacxx: CUDA C++ standard library (atomics, barriers, memory)
- cudax: experimental features (memory resources, allocators)
- Tuning policies: per-SM hardware-specific algorithm parameters

Competition optimization vectors mapped to CCCL:
- Output TPS (83% weight): warp_reduce, block_reduce, device_topk
- Input TPS (14% weight): device_scan, block_load, prefetch
- Cache TPS (3% weight): prefix caching strategy patterns
- Memory (0.9 util): pooled/cached/buddy allocators

Source: https://github.com/NVIDIA/cccl (shallow clone, HEAD only)
License: Apache-2.0
2026-07-30 09:35:51 +00:00

104 lines
2.5 KiB
Plaintext

//===----------------------------------------------------------------------===//
//
// Part of CUDA Experimental in CUDA C++ Core Libraries,
// under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
// SPDX-FileCopyrightText: Copyright (c) 2026 NVIDIA CORPORATION & AFFILIATES.
//
//===----------------------------------------------------------------------===//
#ifndef COMMON_GROUP_CUH
#define COMMON_GROUP_CUH
#include <cuda/barrier>
#include <cuda/std/cstddef>
#include <cuda/std/type_traits>
#include <cuda/warp>
#include <cuda/experimental/group.cuh>
#include "testing.cuh"
namespace
{
template <class T, cuda::std::size_t Id>
__device__ T global_barriers_storage;
//! @brief Returns reference to an array of N cuda::barrier objects with suitable thread scope for level allocated in
//! suitable address space (shared or device memory). Id parameter can be used to create unique object.
template <cuda::std::size_t N, cuda::std::size_t Id = 0, class Level>
__device__ auto& get_barriers(const Level& level) noexcept
{
constexpr auto scope = cudax::__minimum_required_scope_for<Level>();
using Barrier = cuda::barrier<scope>;
using BarriersStorage = cuda::std::aligned_storage_t<N * sizeof(Barrier), alignof(Barrier)>;
if constexpr (scope >= cuda::thread_scope_block)
{
__shared__ BarriersStorage shared_barriers_storage;
return reinterpret_cast<Barrier(&)[N]>(shared_barriers_storage);
}
else
{
return reinterpret_cast<Barrier(&)[N]>(global_barriers_storage<BarriersStorage, Id>);
}
}
struct ThreadsInWarpMappingResult
{
__device__ static constexpr ::cuda::std::size_t static_group_count()
{
return 1;
}
__device__ unsigned group_count() const
{
return 1;
}
__device__ unsigned group_rank() const
{
return 0;
}
__device__ static constexpr ::cuda::std::size_t static_unit_count()
{
return 32;
}
__device__ unsigned unit_count() const
{
return 32;
}
__device__ unsigned unit_rank() const
{
return cuda::gpu_thread.rank_as<unsigned>(cuda::warp);
}
__device__ cuda::device::lane_mask lane_mask() const noexcept
{
return cuda::device::lane_mask::all();
}
__device__ bool is_valid() const
{
return true;
}
__device__ static constexpr bool is_always_exhaustive() noexcept
{
return true;
}
__device__ static constexpr bool is_always_contiguous() noexcept
{
return true;
}
};
} // namespace
#endif // COMMON_GROUP_CUH